'' =================================================================================================
''                 ************** Pressure Testing **********************


''            
''


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000

  REFRESH_RATE = 16_000_000' 80_000_000/5
  

con

  SCL    = 28             'I2C connections to pressure sensor - MS4515DO
  SDA    = 29                                                                                  
  id     = $51            'Device Read address  ---->provided by OEM

  data   = 13             'data in pin to pin 1 of MAX-7219
  clock  = 11             'clock signal in pin to pin 13 of MAX-7219
  load   = 12             'load pulse in pin to pin 12 of MAX-7219
  DP     = %10000000      'decimal point 
  minus  = %1010          'MAX-7219 display code for a minus (-).

  rxpin     = 19             'Receive pin on the Propeller   ---> Datalogger
  txpin     = 18             'Transmit pin on the Propeller
  ctspin    = 17

  
  
  RevPin          = 6
  Baud            = 19200
  Lines           = 2
  INPIN           = 8      'RPM counter PIN from BLDC controller
  MotorPin        = 1      'Motor run/stop
  ExpSolenoid1     = 23     'Exhalation circuit solenoid  connected tp P 23 ---> also the pressure relief valve 1
  ExpSolenoid2     = 22     'Exhalation circuit solenoid connected to P 22 ----> also the pressure relief valve 2
  IntakeSolenoids = 21      'Opens in inspiration and closes during expiration
  Vac              = 14
  
obj

  I2C  : "pasm_i2c_driver"
  Revs : "Debug_Lcd"       
  Seg_7: "7_Seg_LED.Driver_V1.3"
 
  term : "fullduplexserial"
   USB : "Autodatalogger"
                       
var
  
     long RevCounterStack[100]
     long DisplayStack[100]
     long ControlStack[100]
     long waitUntil, inFreq
     long RPM 
    
     long pressure, pressure1
     long filenameaddr
     long num0, num1, num2, num3

     long DataLoggerStack[100]
     
pub main



   I2C.Initialize(SCL)                          ' setup I2C buss (boot pins)
   Revs.init(REVpin, baud, lines)               ' start Serial_LCD
   Seg_7.Start(12,11,13)                        'start 7 seg driver
   pause(10)

   cognew(DataLogger,@DataLoggerStack)                                               
   cognew(Revolution_Counter,@RevCounterStack)
   cognew(ValveControl,@ControlStack)
    
   
   pause(10)
   dira[MotorPin]~~
   pause(10)
   outa[MotorPin]~                       'Start BLDC motor
   
  
  Read_Display_Pressure
  
  
  

Pub Read_Display_Pressure              'READ AND DISPLAY PRESSURE ON THE 8 SEGMENT led DISPLAY
  
  repeat
    Seg_7.Clear(data,clock,load)              'Clear all digits - 7SegLED display
    
              'read I2C
    I2C.Start(SCL)
    I2C.Write(SCL,$51)                        'write read address
    Pressure := I2C.Read(SCL,1)                  'read pressure value to variable "Value"
    Pressure1:= Pressure-31                          ' Error correction(31)on initial reading at atmospheric pressure

     Seg_7.CharOut(data,clock,load,Pressure1)      'send pressure data to 7Seg.LED
     
     pause(50)
     
  

     
Pub ValveControl                           'CONTROL SOLENOID VALVES DURING INSPIRATORY & EXPIRATORY PHASE

    dira[21..23]~~                         'Set solenoid pins as outputs
    dira[14]~~
    outa[23]~~
 repeat
 
       If Pressure1  < 0                
           outa[21]~~
           outa[22]~
           outa[23]~
           outa[14]~                      'Inspiratory phase Pump valve open/bypass valve closed and expiratory valves close 
          'pause(10)

       ElseIf pressure1 > 4
           outa[21]~                     ' Expiratory phase ---> pump valve closes/bypass valve opens.
           outa[22]~~
           outa[23]~~
           outa[14]~~                    'BOTH Relief valves @22&23 open
           'pause(10)
           
   If Pressure1 > 3             'In case of inadvertant pressure rises > 5cm.H2O - pump is bypassed and
        outa[22..23]~~
        outa[14]~~                 ' both pressure relief valves opne to AIR
        pause(200)
 
Pub DataLogger                             'DATA LOGGER CODE

  filenameaddr := string("Pressure.txt")
  num1 := num2 := num3 := 0

  waitcnt(clkfreq*6 + cnt)

  waitcnt(clkfreq*2 + cnt)

  USB.init(rxpin, txpin, ctspin, filenameaddr, 10_000)



  
  '           (address, digits, title)
  USB.addfield(@pressure, 2, string("Raw"))
  USB.addfield(@RPM, 4, string("RPM"))
  USB.addfield(@Pressure1, 2, string("corrected"))
                                                    '
   USB.start

  

  repeat num0 from 1 to 2
    num0[num0] := 3_999_999_999

  num0 := cnt 

  repeat 1
    
    repeat 60                 '1 Minute
      
      waitcnt(num0 += clkfreq)  'Seconds
     
  USB.stop

  repeat



   


Pub Revolution_Counter

   Revs.backLight(true)                                  
   Revs.cursor(0) 

  ctra := %01010 << 26 + INPIN    ' counter A in POSedge mode
  frqa := 1                       ' increment 1 per pulse  waitcnt(clkfreq + cnt)

' measure the input duty cycle using counters.
  waitUntil := cnt + REFRESH_RATE
  repeat
    phsa := 0                     ' reset count registers
    waitcnt(waitUntil+=REFRESH_RATE)
    inFreq       := phsa * (constant(80_000_000/REFRESH_RATE))         ' read the number of edges; convert to 1 second
    RPM:=(15*inFreq)
    Revs.CLS
    Revs.str(string("RPM :"))
    'waitcnt(clkfreq / 200 + cnt)
    Revs.dec(RPM)


        
pub pause(ms) | t

'' Delay program in milliseconds

  t := cnt - 1088                              ' sync with system counter
  repeat (ms #> 0)
    waitcnt(t += MS_001)
    
 
dat

{{

  Copyright (c) 2009-10 Jon McPhalen (aka Jon Williams)  AND Autodataloggeer V 1.0  By SRLM 



  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}

                   